# Copyright (c) 2011, Intel Corporation
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of Intel Corporation nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""testsuite module."""

import bits

verbose = 0
passed, failed = 0, 0

def CSR(name, uncore_bus_num, dev, fun, reg, bytes=4, highbit=63, lowbit=0):
    if bytes == 4:
        highbit = min(highbit, 31)
    elif bytes == 2:
        highbit = min(highbit, 15)
    elif bytes == 1:
        highbit = min(highbit, 7)
    value = (bits.pcie_read(uncore_bus_num, dev, fun, reg, bytes) & ((1 << (highbit + 1)) - 1)) >> lowbit
    detail = "{0} (CSR B{1:x}h:D{2}:F{3}:{4:x}h [{5:d}:{6:d}])".format(name, uncore_bus_num, dev, fun, reg, highbit, lowbit)
    detail += " = 0x{0:x}".format(value)
    return value, detail

def MSR(name, apicid, msr, highbit=63, lowbit=0):
    value = (bits.rdmsr(apicid, msr) & ((1 << (highbit + 1)) - 1)) >> lowbit
    if highbit == 63 and lowbit == 0:
        detail = "{0} (MSR {1:x}h)".format(name, msr)
    else:
        detail = "{0} (MSR {1:x}h [{2:d}:{3:d}])".format(name, msr, highbit, lowbit)
    detail += " = 0x{0:x}".format(value)
    return value, detail

def get_summary_count():
    global passed, failed
    return (passed, failed)

def init():
    global passed, failed
    passed, failed = 0, 0

def rdmsr_consistent(msr_blacklist=set(), msr_masklist=dict()):
    """Rdmsr for all CPU and verify consistent value"""

    cpulist = sorted(bits.cpus())
    for r in [range(0, 0x1000), range(0xC0000000, 0xC0001000)]:
        for msr in r:
            if msr in msr_blacklist:
                continue
            mask = msr_masklist.get(msr, ~0)
            uniques = {}
            for cpu in cpulist:
                value = bits.rdmsr(cpu, msr)
                if value is not None:
                    value &= mask
                uniques.setdefault(value, []).append(cpu)
            if test("MSR 0x{0:x} consistent".format(msr), len(uniques) == 1):
                print_detail("{0} unique values".format(len(uniques)))
                for value, cpus in uniques.iteritems():
                    print_detail("{0} CPUs: {1}".format(len(cpus), ",".join(str(c) for c in cpus)))
                    if value is None:
                        print_detail("MSR 0x{0:x}: GPF".format(msr))
                    else:
                        print_detail("MSR 0x{0:x}: 0x{1:x}".format(msr, value))

def test(desc, value):
    """Test a condition; pass if bool(value) is True. Returns True if the caller should print more detail."""
    global passed, failed
    condition = bool(value)
    if condition:
        passed += 1
    else:
        failed += 1
    if not(condition) or verbose:
        print "[assert] {0} {1}".format(desc, pass_fail_str(condition))
        return True
    return False

def pass_fail_str(condition):
    if condition:
        return 'PASS'
    return 'FAIL'

def print_detail(str):
    print '  {0}'.format(str)

def print_summary():
    global passed, failed
    print 'Passed={0}; Failed={1}'.format(passed, failed)
